function [dens] = gaussian_density(DATA, MEANS, SIGS, PRIORS)
%
% Computes the density of each datapoint under each of the Gaussian classes.
%
% ---- Preconditions ---- 
% DATA: rows contain multidimensional datapoints (d columns)
% MEANS: the ith row is the mean of the ith model
% SIGS:  array of covariance matrices sigs(:,:,i) is the cov of ith model
% PRIOR: column vector with the prior of the ith model in the ith row.
% 
% ---- Postconditions ----
% dens: a column vector where the nth row contains the density associated
%       with the datapoint in the nth row of data.
%
% ----- Description -----
% This function computes a density from a multigaussian model, where each
% gaussian has a different mean, covariance, and prior probability. In the
% case of one gaussian, the function will also work, naturally.


[N,D] = size(DATA); % N <- number of datapoints, D <- dimensionality
M = size(MEANS, 1);
dens = zeros(N,M);
for m=1:M,
  const = 1 / (sqrt((2*pi)^D) * sqrt(det(SIGS(:,:,m)))); 
  % Tricky..  we want the sqrt of the inverse.
  invsig = SIGS(:,:,m)^-0.5;
  % Subtract mean
  cdata = DATA - ones(N,1)*MEANS(m,:);
  % Multiply each row by inv. cov.
  ndata = cdata * invsig; 
  % Take the componentwise product
  pdata = ndata .* ndata;
  % Take the row sum of the componentwise products
  dots = sum(pdata')';
  dens(:,m) = const * PRIORS(m) * exp(-0.5 * dots);
end;


